"""punter - A simple wrapper for the Email Hunter API."""


__title = 'punter'
__author__ = 'Joshua Goodlett'
__version__ = '0.1.2'
__license__ = 'MIT'


from .api import search
from .exceptions import (
    PunterException,
    InvalidAPIKeyException,
    InvalidQueryStringException
)



"""Essential punter API."""


import json
import requests
from .exceptions import PunterException, InvalidAPIKeyException
from .helpers import get_endpoint


def search(api_key, query, offset=0, type='personal'):
    """Get a list of email addresses for the provided domain.

    The type of search executed will vary depending on the query provided.
    Currently this query is restricted to either domain searches, in which
    the email addresses (and other bits) for the domain are returned, or
    searches for an email address. The latter is primary meant for checking
    if an email address exists, although various other useful bits are also
    provided (for example, the domain where the address was found).

    :param api_key: Secret client API key.
    :param query: URL or email address on which to search.
    :param offset: Specifies the number of emails to skip.
    :param type: Specifies email type (i.e. generic or personal).

    """

    if not isinstance(api_key, str):
        raise InvalidAPIKeyException('API key must be a string')

    if not api_key or len(api_key) < 40:
        raise InvalidAPIKeyException('Invalid API key.')

    url = get_endpoint(api_key, query, offset, type)

    try:
        return requests.get(url).json()
    except requests.exceptions.RequestException as err:
        raise PunterException(err)



"""Custom exceptions for the punter package."""


class PunterException(Exception):
    def __init__(self, *args, **kwargs):
        super(PunterException, self).__init__(*args, **kwargs)


class InvalidAPIKeyException(PunterException):
    pass


class InvalidQueryStringException(PunterException):
    pass



"""A collection of helper/utility functions."""


import re
from . import exceptions as ex


API_ENDPOINT = 'https://api.emailhunter.co/v1/'
EMAIL_URL = API_ENDPOINT + 'exist?email={0}&api_key={1}'
DOMAIN_URL = (API_ENDPOINT + 'search?domain={0}&api_key={1}'
              '&offset={2}&type={3}')


EMAIL_RE = re.compile(
    '''
    [\w\d.+-]+             # username
    @
    ([\w\d.]+\.)+          # domain name prefix
    (com|org|edu|io|me)    # support more top-level domains
    ''',
    re.UNICODE | re.VERBOSE | re.IGNORECASE)


# From Django, slightly modified (http://bit.ly/1ILBmfL)
URL_RE = re.compile(
    '''
    ^(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+
    (?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|             # domain...
    localhost|                                      # localhost...
    \d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|             # ...or ipv4
    \[?[A-F0-9]*:[A-F0-9:]+\]?)                     # ...or ipv6
    (?::\d+)?                                       # optional port
    (?:/?|[/?]\S+)$
    ''',
    re.VERBOSE | re.IGNORECASE)


def get_query_type(query):
    """Gets the type of query in use (email or url).

    In order to provide a proper API endpoint it is necessary to first
    determine the type of query the client is using. There are only two
    options currently: 1) domain or 2) email.

    :param query: Search query provided by client.

    """

    if URL_RE.match(query):
        query_type = 'domain'
    elif EMAIL_RE.match(query):
        query_type = 'email'
    else:
        query_type = ''

    return query_type


def get_endpoint(api_key, query, offset, type):
    """Return endpoint URL for the relevant search type.

    The base API endpoint only varies by type of search requested, of
    which there are two: 1) domain search and 2) email search. Each
    search type requires different parameters, though api_key is common
    between them.

    Note: if both a url and email address are provided the endpoint
    returned will default to the domain search as it is considered to
    be the primary function of the API and thus takes precedent.

    :param api_key: Secret client API key.
    :param query: URL or email address on which to search.
    :param offset: Specifies the number of emails to skip.
    :param type: Specifies email type (i.e. generic or personal).

    """

    query_type = get_query_type(query)

    if query_type not in ('domain', 'email'):
        raise ex.InvalidQueryStringException('Invalid query string')

    if query_type == 'domain':
        return DOMAIN_URL.format(query, api_key, offset, type)
    else:
        return EMAIL_URL.format(query, api_key)



#!/usr/bin/env python

import re
from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
    long_description = open('README.md').read()

version = ''
with open('punter/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')


setup(
    name='punter',
    version=version,
    description='Python wrapper for the Email Hunter API',
    long_description=long_description,
    author='Joshua Goodlett',
    author_email='joshuagoodlett@gmail.com',
    url='https://github.com/jgoodlet/punter',
    license='MIT',
    packages=find_packages(),
    tests_require=['pytest'],
    install_requires=['requests'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ]
)